@@ -1,8 +1,9 @@
+skip_on_cran()
+skip_if_offline(url_parse(hb())$domain)
+
 context("Async - General")
 
 test_that("Async works", {
-  skip_on_cran()
-
   expect_is(Async, "R6ClassGenerator")
 
   aa <- Async$new(urls = c(hb('/get'), 'https://google.com'))
@@ -22,7 +23,7 @@ test_that("Async works", {
 
   # after calling
   res <- aa$get()
-  expect_is(res, "list")
+  expect_is(res, "asyncresponses")
   expect_equal(length(res), 2)
   expect_is(res[[1]], "HttpResponse")
   expect_is(res[[1]]$request, "HttpRequest")
@@ -30,14 +31,10 @@ test_that("Async works", {
 })
 
 test_that("Async fails well", {
-  skip_on_cran()
-
   expect_error(Async$new(), "\"urls\" is missing, with no default")
 })
 
 test_that("Async print method", {
-  skip_on_cran()
-  
   aa <- Async$new(urls = c(hb('/get'), 'https://google.com'))
 
   expect_is(aa, "Async")
@@ -60,7 +57,6 @@ test_that("Async print method", {
 
 
 test_that("Async curl options work", {
-  skip_on_cran()
   skip_on_ci() # not sure why, but not working on CI
   
   aa <- Async$new(urls = c(hb('/get'), 'https://google.com'), 
@@ -72,8 +68,6 @@ test_that("Async curl options work", {
 })
 
 test_that("Async headers work", {
-  skip_on_cran()
-  
   aa <- Async$new(urls = c(hb('/get'), 'https://google.com'), 
     headers = list(foo = "bar"))
   expect_output(aa$print(), "headers")
@@ -87,13 +81,11 @@ test_that("Async headers work", {
 
 context("Async - get")
 test_that("Async - get", {
-  skip_on_cran()
-
   aa <- Async$new(urls = c(hb('/get'),
                            'https://google.com'))
   out <- aa$get()
 
-  expect_is(out, "list")
+  expect_is(out, "asyncresponses")
   expect_is(out[[1]], "HttpResponse")
   expect_is(out[[2]], "HttpResponse")
   expect_equal(out[[1]]$method, "get")
@@ -103,13 +95,11 @@ test_that("Async - get", {
 
 context("Async - post")
 test_that("Async - post", {
-  skip_on_cran()
-
   aa <- Async$new(urls = c(hb('/post'),
                            hb('/post')))
   out <- aa$post()
 
-  expect_is(out, "list")
+  expect_is(out, "asyncresponses")
   expect_is(out[[1]], "HttpResponse")
   expect_equal(out[[1]]$method, "post")
 })
@@ -117,13 +107,11 @@ test_that("Async - post", {
 
 context("Async - put")
 test_that("Async - put", {
-  skip_on_cran()
-
   aa <- Async$new(urls = c(hb('/put'),
                            hb('/put')))
   out <- aa$put()
 
-  expect_is(out, "list")
+  expect_is(out, "asyncresponses")
   expect_is(out[[1]], "HttpResponse")
   expect_is(out[[2]], "HttpResponse")
   expect_equal(out[[1]]$method, "put")
@@ -133,13 +121,11 @@ test_that("Async - put", {
 
 context("Async - patch")
 test_that("Async - patch", {
-  skip_on_cran()
-
   aa <- Async$new(urls = c(hb('/patch'),
                            hb('/patch')))
   out <- aa$patch()
 
-  expect_is(out, "list")
+  expect_is(out, "asyncresponses")
   expect_is(out[[1]], "HttpResponse")
   expect_is(out[[2]], "HttpResponse")
   expect_equal(out[[1]]$method, "patch")
@@ -149,13 +135,11 @@ test_that("Async - patch", {
 
 context("Async - delete")
 test_that("Async - delete", {
-  skip_on_cran()
-
   aa <- Async$new(urls = c(hb('/delete'),
                            hb('/delete')))
   out <- aa$delete()
 
-  expect_is(out, "list")
+  expect_is(out, "asyncresponses")
   expect_is(out[[1]], "HttpResponse")
   expect_is(out[[2]], "HttpResponse")
   expect_equal(out[[1]]$method, "delete")
@@ -165,13 +149,11 @@ test_that("Async - delete", {
 
 context("Async - head")
 test_that("Async - head", {
-  skip_on_cran()
-
   aa <- Async$new(urls = c('https://google.com',
                            'https://nytimes.com'))
   out <- aa$head()
 
-  expect_is(out, "list")
+  expect_is(out, "asyncresponses")
   expect_is(out[[1]], "HttpResponse")
   expect_is(out[[2]], "HttpResponse")
   expect_equal(out[[1]]$method, "head")
@@ -180,30 +162,41 @@ test_that("Async - head", {
 
 context("Async - verb")
 test_that("Async - verb", {
-  skip_on_cran()
-
   aa <- Async$new(urls = c('https://google.com',
                            'https://nytimes.com'))
   out <- aa$verb('get')
 
-  expect_is(out, "list")
+  expect_is(out, "asyncresponses")
   expect_is(out[[1]], "HttpResponse")
   expect_is(out[[2]], "HttpResponse")
   expect_equal(out[[1]]$method, "get")
   expect_equal(out[[2]]$method, "get")
 })
 
+context("Async - verb")
+test_that("Async - retry", {
+  aa <- Async$new(urls = c("https://nghttp2.org/httpbin/status/404", 
+    "https://nghttp2.org/httpbin/status/429"))
+  out <- aa$retry(verb='get')
+
+  expect_is(out, "asyncresponses")
+  expect_is(out[[1]], "HttpResponse")
+  expect_is(out[[2]], "HttpResponse")
+  expect_equal(out[[1]]$method, "get")
+  expect_equal(out[[2]]$method, "get")
+  expect_gt(length(out[[1]]$response_headers_all), 3)
+  expect_gt(length(out[[2]]$response_headers_all), 3)
+})
+
 
 context("Async - order of results")
 test_that("Async - order", {
-  skip_on_cran()
-
   aa <- Async$new(urls = c(hb('/get?a=5'),
                            hb('/get?b=6'),
                            hb('/get?c=7')))
   out <- aa$get()
 
-  expect_is(out, "list")
+  expect_is(out, "asyncresponses")
   expect_is(out[[1]], "HttpResponse")
   expect_is(out[[2]], "HttpResponse")
   expect_is(out[[3]], "HttpResponse")
@@ -215,8 +208,6 @@ test_that("Async - order", {
 
 context("Async - disk w/ GET")
 test_that("Async - writing to disk works", {
-  skip_on_cran()
-
   cc <- Async$new(
     urls = c(
       hb('/get?a=5'),
@@ -233,7 +224,7 @@ test_that("Async - writing to disk works", {
   # cleanup
   closeAllConnections()
 
-  expect_is(res, "list")
+  expect_is(res, "asyncresponses")
   expect_is(res[[1]], "HttpResponse")
   expect_is(out, "list")
   expect_is(out[[1]], "character")
@@ -241,8 +232,6 @@ test_that("Async - writing to disk works", {
 
 context("Async - disk w/ POST")
 test_that("Async - writing to disk works", {
-  skip_on_cran()
-
   post_url <- hb('/post')
   cc <- Async$new(urls = rep(post_url, 5))
   files <- replicate(5, tempfile())
@@ -252,7 +241,7 @@ test_that("Async - writing to disk works", {
   # cleanup
   closeAllConnections()
 
-  expect_is(res, "list")
+  expect_is(res, "asyncresponses")
   expect_is(res[[1]], "HttpResponse")
   expect_is(out, "list")
   expect_is(out[[1]], "character")
@@ -261,8 +250,6 @@ test_that("Async - writing to disk works", {
 
 context("Async - disk w/ PUT")
 test_that("Async - writing to disk works", {
-  skip_on_cran()
-
   put_url <- hb('/put')
   cc <- Async$new(urls = rep(put_url, 5))
   files <- replicate(5, tempfile())
@@ -272,7 +259,7 @@ test_that("Async - writing to disk works", {
   # cleanup
   closeAllConnections()
 
-  expect_is(res, "list")
+  expect_is(res, "asyncresponses")
   expect_is(res[[1]], "HttpResponse")
   expect_is(out, "list")
   expect_is(out[[1]], "character")
@@ -281,8 +268,6 @@ test_that("Async - writing to disk works", {
 
 context("Async - disk w/ PATCH")
 test_that("Async - writing to disk works", {
-  skip_on_cran()
-
   patch_url <- hb('/patch')
   cc <- Async$new(urls = rep(patch_url, 5))
   files <- replicate(5, tempfile())
@@ -292,7 +277,7 @@ test_that("Async - writing to disk works", {
   # cleanup
   closeAllConnections()
 
-  expect_is(res, "list")
+  expect_is(res, "asyncresponses")
   expect_is(res[[1]], "HttpResponse")
   expect_is(out, "list")
   expect_is(out[[1]], "character")
@@ -301,8 +286,6 @@ test_that("Async - writing to disk works", {
 
 context("Async - disk w/ DELETE")
 test_that("Async - writing to disk works", {
-  skip_on_cran()
-
   delete_url <- hb('/delete')
   cc <- Async$new(urls = rep(delete_url, 5))
   files <- replicate(5, tempfile())
@@ -312,7 +295,7 @@ test_that("Async - writing to disk works", {
   # cleanup
   closeAllConnections()
 
-  expect_is(res, "list")
+  expect_is(res, "asyncresponses")
   expect_is(res[[1]], "HttpResponse")
   expect_is(out, "list")
   expect_is(out[[1]], "character")
@@ -321,8 +304,6 @@ test_that("Async - writing to disk works", {
 
 context("Async - stream")
 test_that("Async - streaming to disk works", {
-  skip_on_cran()
-
   bb <- Async$new(urls = c(hb('/get?a=5'),
                            hb('/get?b=6'),
                            hb('/get?c=7')))
@@ -348,8 +329,6 @@ test_that("Async - streaming to disk works", {
 
 context("Async - basic auth")
 test_that("Async - with basic auth works", {
-  skip_on_cran()
-
   dd <- Async$new(
     urls = rep(hb('/basic-auth/user/passwd'), 3), 
     auth = auth(user = "user", pwd = "passwd")
@@ -371,13 +350,11 @@ test_that("Async - with basic auth works", {
 
 context("Async - failure behavior w/ bad URLs/etc.")
 test_that("Async - failure behavior", {
-  skip_on_cran()
-
   urls <- c("http://stuffthings.gvb", "https://foo.com", "https://scottchamberlain.info")
   conn <- Async$new(urls = urls)
   res <- conn$get()
 
-  expect_is(res, "list")
+  expect_is(res, "asyncresponses")
   
   expect_is(res[[1]], "HttpResponse")
   expect_is(res[[2]], "HttpResponse")
@@ -396,14 +373,12 @@ test_that("Async - failure behavior", {
 
 context("Async - failure behavior w/ bad URLs/etc. - disk")
 test_that("Async - failure behavior", {
-  skip_on_cran()
-
   files <- replicate(3, tempfile())
   urls <- c("http://stuffthings.gvb", "https://foo.com", "https://scottchamberlain.info")
   conn <- Async$new(urls = urls)
   res <- conn$get(disk = files)
 
-  expect_is(res, "list")
+  expect_is(res, "asyncresponses")
   
   expect_is(res[[1]], "HttpResponse")
   expect_is(res[[2]], "HttpResponse")
@@ -419,7 +394,7 @@ test_that("Async - failure behavior", {
 
   expect_match(res[[1]]$parse("UTF-8"), "resolve host")
   expect_is(res[[2]]$parse("UTF-8"), "character")
-  expect_match(res[[3]]$parse("UTF-8"), "DOCTYPE")
+  expect_match(res[[3]]$parse("UTF-8"), "doctype")
 
   expect_equal(length(readLines(files[1])), 0)
   expect_equal(length(readLines(files[2])), 0)
@@ -431,8 +406,6 @@ test_that("Async - failure behavior", {
 
 context("Async - failure behavior w/ bad URLs/etc. - stream")
 test_that("Async - failure behavior", {
-  skip_on_cran()
-
   mylist <- c()
   fun <- function(x) mylist <<- append(mylist, list(x))
 
@@ -440,7 +413,7 @@ test_that("Async - failure behavior", {
   conn <- Async$new(urls = urls)
   res <- conn$get(stream = fun)
 
-  expect_is(res, "list")
+  expect_is(res, "asyncresponses")
   
   expect_is(res[[1]], "HttpResponse")
   expect_is(res[[2]], "HttpResponse")
