@@ -1,8 +1,8 @@
+skip_on_cran()
+skip_if_offline(url_parse(hb())$domain)
 context("AsyncVaried")
 
 test_that("AsyncVaried works", {
-  skip_on_cran()
-
   expect_is(AsyncVaried, "R6ClassGenerator")
 
   req1 <- HttpRequest$new(url = hb("/get"))$get()
@@ -37,16 +37,12 @@ test_that("AsyncVaried works", {
 })
 
 test_that("AsyncVaried fails well", {
-  skip_on_cran()
-
   expect_error(AsyncVaried$new(), "must pass in at least one request")
   expect_error(AsyncVaried$new(5), "all inputs must be of class 'HttpRequest'")
 })
 
 context("AsyncVaried - order of results")
 test_that("AsyncVaried - order", {
-  skip_on_cran()
-
   req1 <- HttpRequest$new(url = hb("/get?a=5"))$get()
   req2 <- HttpRequest$new(url = hb("/get?b=6"))$get()
   req3 <- HttpRequest$new(url = hb("/get?c=7"))$get()
@@ -67,8 +63,6 @@ test_that("AsyncVaried - order", {
 
 context("AsyncVaried - disk")
 test_that("AsyncVaried - writing to disk works", {
-  skip_on_cran()
-
   f <- tempfile()
   g <- tempfile()
   req1 <- HttpRequest$new(url = hb("/get"))$get(disk = f)
@@ -103,8 +97,6 @@ test_that("AsyncVaried - writing to disk works", {
 
 context("AsyncVaried - stream")
 test_that("AsyncVaried - streaming to disk works", {
-  skip_on_cran()
-
   lst <- c()
   fun <- function(x) lst <<- append(lst, list(x))
   req1 <- HttpRequest$new(url = hb("/get")
@@ -129,8 +121,6 @@ test_that("AsyncVaried - streaming to disk works", {
 
 context("AsyncVaried - basic auth")
 test_that("AsyncVaried - basic auth works", {
-  skip_on_cran()
-
   url <- hb("/basic-auth/user/passwd")
   auth <- auth(user = "user", pwd = "passwd")
   reqlist <- list(
@@ -155,8 +145,6 @@ test_that("AsyncVaried - basic auth works", {
 
 context("AsyncVaried - failure behavior w/ bad URLs/etc.")
 test_that("AsyncVaried - failure behavior", {
-  skip_on_cran()
-
   reqlist <- list(
     HttpRequest$new(url = "http://stuffthings.gvb")$get(),
     HttpRequest$new(url = base_url)$head(),
@@ -178,15 +166,12 @@ test_that("AsyncVaried - failure behavior", {
   expect_false(resps[[3]]$success())
 
   expect_match(resps[[1]]$parse("UTF-8"), "resolve host")
-  expect_true(grepl("time", resps[[3]]$parse("UTF-8"), ignore.case = TRUE))
 })
 
 
 # disk and stream behave the same was as w/o either of them
 context("AsyncVaried - failure behavior w/ bad URLs/etc. - disk")
 test_that("AsyncVaried - failure behavior", {
-  skip_on_cran()
-
   f <- tempfile()
   g <- tempfile()
   reqlist <- list(
@@ -207,7 +192,6 @@ test_that("AsyncVaried - failure behavior", {
   expect_false(resps[[2]]$success())
 
   expect_match(resps[[1]]$parse("UTF-8"), "resolve host")
-  expect_true(grepl("time", resps[[2]]$parse("UTF-8"), ignore.case = TRUE))
   
   # cleanup
   closeAllConnections()
@@ -216,8 +200,6 @@ test_that("AsyncVaried - failure behavior", {
 # disk and stream behave the same was as w/o either of them
 context("AsyncVaried - failure behavior w/ bad URLs/etc. - stream")
 test_that("AsyncVaried - failure behavior", {
-  skip_on_cran()
-
   lst <- c()
   fun <- function(x) lst <<- c(lst, x)
   reqlist <- list(
@@ -238,13 +220,10 @@ test_that("AsyncVaried - failure behavior", {
   expect_false(resps[[2]]$success())
 
   expect_match(resps[[1]]$parse("UTF-8"), "resolve host")
-  expect_true(grepl("time", resps[[2]]$parse("UTF-8"), ignore.case = TRUE))
 })
 
 # verb method works
 test_that("AsyncVaried verb method works", {
-  skip_on_cran()
-
   req1 <- HttpRequest$new(url = hb("/get"))$verb('get')
   req2 <- HttpRequest$new(url = hb("/post"))$verb('post')
 
@@ -253,3 +232,14 @@ test_that("AsyncVaried verb method works", {
   expect_equal(aa$responses()[[1]]$method, 'get')
   expect_equal(aa$responses()[[2]]$method, 'post')
 })
+
+# verb method works
+test_that("AsyncVaried verb retry", {
+  req1 <- HttpRequest$new(url = hb("/get"))$retry('get')
+  req2 <- HttpRequest$new(url = hb("/post"))$retry('post')
+
+  aa <- AsyncVaried$new(req1, req2)
+  aa$request()
+  expect_equal(aa$responses()[[1]]$method, 'get')
+  expect_equal(aa$responses()[[2]]$method, 'post')
+})
