@@ -1,8 +1,8 @@
+skip_on_cran()
+skip_if_offline(url_parse(hb())$domain)
 context("HttpClient: head")
 
 test_that("head request works", {
-  skip_on_cran()
-
   cli <- HttpClient$new(url = "https://www.google.com")
   aa <- cli$head()
 
@@ -21,8 +21,6 @@ test_that("head request works", {
 
 
 test_that("head - query passed to head doesn't fail", {
-  skip_on_cran()
-
   cli <- HttpClient$new(url = "https://www.google.com")
   aa <- cli$head(query = list(foo = "bar"))
 
@@ -39,3 +37,15 @@ test_that("head - query passed to head doesn't fail", {
   # content is empty
   expect_equal(aa$content, raw(0))
 })
+
+
+test_that("with auth works", {
+  cli <- HttpClient$new(url = hb(), auth = auth("foo", "bar"))
+  aa <- cli$head("/basic-auth/foo/bar")
+
+  expect_is(aa, "HttpResponse")
+  expect_equal(aa$method, "head")
+  expect_true(aa$success())
+  expect_equal(aa$status_code, 200)
+  expect_equal(aa$request$options$userpwd, "foo:bar")
+})
